This specification outlines the work that is required to use Gradle to build applications that use the [Play framework](http://www.playframework.com).

# Use cases

There are 3 main use cases:

- A developer builds a Play application.
- A developer runs a Play application during development.
- A deployer runs a Play application. That is, a Play application is packaged up as a distribution which can be run in a production environment.

# Out of scope

The following features are currently out of scope for this spec, but certainly make sense for later work:

- Building a Play application for multiple Scala versions. For now, the build for a given Play application will target a single Scala version.
  It will be possible to declare which version of Scala to build for.
- Using continuous mode with JVMs older than Java 7. For now, this will work only with Java 7 and later will be supported. It will be possible to build and run for Java 6.
- Any specific IDE integration, beyond Gradle's current general purpose IDE integration for Java and Scala.
- Any specific testing support, beyond Gradle's current support for testing Java and Scala projects.
- Any specific support for publishing and resolving Play applications, beyond Gradle's current general purpose capabilities.
- Any specific support for authoring plugins, beyond Gradle's current support.
- Installing the Play tools on the build machine.
- Migrating or importing SBT settings for a Play project.

# Performance

Performance should be comparable to SBT:

- Building and starting an application.
- Reload after a change.
- Executing tests for an application.

# Milestone 1

For the first milestone, a developer will be able to define, build and run a simple play application that consists
of routes and templates, Java and Scala sources, as well as Javascript, CSS, CoffeeScript and LESSCSS sources.

In this milestone we are not dealing with:
- Automatically rebuilding the play application when sources change
- 3rd party dependencies for Java, Scala or Javascript. (This may change).

The goal is to get something working pretty quickly, and work to improve the configurability and discoverability over time.

## Feature: Developer builds, tests and runs a basic Play application

### Story: Build author declares and builds a Play application component (✔︎)

Add a `play-application` plugin that provides Play application component:

```gradle
plugins {
    id 'play-application'
}

model {
    components {
        myapp(PlayApplicationSpec)
    }
}
```

- ~~Running `gradle assemble` builds an empty Jar file.~~
- ~~Running `gradle components` shows some basic details about the Play application.~~
- In this story, no source files are supported.

#### Test cases

- ~~component report shows PlayApplicationSpec with~~
- ~~version info about~~
      - ~~play  (declared in the plugin)~~
      - ~~java  (picked current version for now)~~
- ~~assemble creates an empty jar file~~

### Story: Developer builds a basic Play application (✔︎)

When running `gradle assemble`, a Jar file will be built for the default template application generated by Play.

- Using hard-coded:
    - Locations of java/scala files
    - Locations of routes file
    - Locations of templates files
    - Scala version
    - Dependencies of Play ("com.typesafe.play:play_2.11:2.3.7")
    - Dependency of template compiler ("com.typesafe.play:twirl-compiler_2.11:1.0.2)
    - Dependency of routes compiler
- resolve different play dependencies from user-configured repositories

#### Implementation

- setup TwirlCompiler task type
- setup RoutesCompiler task type
- Compile routes to scala and java
- Compile templates to scala
- Compile all scala (app/\*/\*.{scala,java}, output of: conf/routes, output of: app/views/\*.scala.html) files
- Output class files are part of assemble jar file

#### Test cases

- verify that generated scala template files exists
- verify that generated scala/java route files exists
- `gradle assemble` should trigger compile task and output jar should contain class files

### Story: Developer runs a basic Play application (✔︎)

Extend the Play support to allow the Play application to be executed.

- Running `gradle assemble` produces an executable Jar that, when executed, runs the Play application.
- Running `gradle run<ComponentName>` builds and executes the Play application.

At this stage, only the default generated Play application is supported, with a hard-coded version of Scala and Play.

#### Test cases

- verify that play app can be built and executed with play version 2.3.7 and 2.2.3
- Can configure port for launched PlayApp: default is 9000

#### Open issue

- Stopping the app with Ctrl-D?
- Testing stopping via keyboard interaction with Daemon (currently test is disabled)

### Story: Developer tests a basic Play application (✔︎)

- Add a `Test` test task per binary that runs the play application tests based on JUnit TestRunner against the binary
- The test sources are compiled against the binary and the `play-test` dependency based on play and scala version of the platform
- Can execute Play unit and integration tests
- Fails with a nice error message
- Wired into the `check` and `test` lifecycle tasks

#### Test cases
- Verify that running `gradle testBinary` and `gradle test` executes tests in the `/test` directory
- Verify output and reports generated for successful tests
- Verify output and reports generated for failing tests

#### Open issues

- Model test suites
- Pull `check` lifecycle task up into `LifecycleBasePlugin` and wire in all test tasks.

### Story: Developer builds, runs and tests a basic application for a specified version of Play (✔︎)

- Can build play application with Play 2.2.3 on Scala 2.10, and Play 2.3.7 on Scala 2.11

#### Test cases

- Verify building and running the 'activator new' app with Play 2.2.3 and Play 2.3.7

#### Open issues

- Compile Scala wrappers around different Play versions, and load these via reflection

### Story: Developer builds Play application distribution (✔︎)

Introduce some lifecycle tasks to allow the developer to package up the Play application. For example, the
developer may run `gradle stage` to stage the local application, or `gradle dist` to create a standalone distribution.

- Build distribution image and zips, as per `play stage` and `play dist`
- Default distributions are added for each play binary
- Developer can add additional content to a default distribution:
```
model {
    distributions {
        playBinary {
            contents {
                from "docs"
            }
        }
    }
}
```


#### Test cases
- Default distributions are added for each play binary
- Stage and Zip tasks are added for each distribution
- Lifecycle tasks for "stage" and "dist" are added to aggregate all distribution tasks
- Distribution base name defaults to distribution name, which defaults to play binary name
- Distribution archive name defaults to "[distribution-base-name]-[project.version].zip"
- Public assets are packaged in a separate jar with a classifier of "assets".
- Default distribution zip contains:
    - all content under a directory named "[distribution-base-name]-[project.version]"
    - jar and assets jar in "lib".
    - all runtime libraries in "lib".
    - application script and batch file in "bin".
    - application.conf and any secondary routes files in "conf".
    - any README file provided in conventional location "${projectDir}/README"
- content added to the distribution is also included in the zip
- application script and batch file will successfully run play:
    - can access a public asset
    - can access a custom route

#### Open Issues
- A Play distribution zip, by default, contains a shared/docs directory with the scaladocs for the application.  We'll need
a scaladoc task wired in to duplicate this functionality.

## Feature: Developer builds Play application with custom Java, Scala, routes and templates

### Story: Developer includes Scala sources in Jvm library (✔︎)

Add a new 'scala-lang' plugin to add Scala language support implemented in the same manner as the JavaLanguagePlugin.

```gradle
plugins {
    id 'jvm-component'
    id 'scala-lang'
}
model {
    components {
        myLib(JvmLibrarySpec)
        myOtherLib(JvmLibrarySpec) {
            sources {
                scala {
                    source.srcDir "src/myOtherLib/myScala"
                }
                otherScala(ScalaSourceSet) {
                    source.srcDir "src/otherScala"
                }
            }
        }
    }
}
```

##### Test cases

- Scala source set(s) and locations are visible in the components report
- Uses scala sources from conventional location
- Can configure location of scala sources
- Build is incremental:
    - Changed in Scala comment does not result in rebuilding JVM library
    - Change in Scala compile properties triggers recompilation
    - Change in Scala platform triggers recompilation
    - Change in Scala ToolChain triggers recompilation
    - Removal of Scala source files removes classes for that source file, which are removed from JVM library
- Compile is incremental: consider sources A, B, C where B depends on C
    - Change A, only A is recompiled
    - Change B, only B is recompiled
    - Change C, only B & C are recompiled
    - Remove A, outputs for A are removed
- Can include both Java and Scala sources in lib: no cross-compilation

### Story: Developer configures Scala sources and Jvm resources for Play application

Extend the Play support to full model Scala sources and Jvm resources.

- A default ScalaLanguageSourceSet 'scala' will include java and scala source files in `app/controllers` and `app/models`.
- A default JvmResourceSet 'resources' will include files in `public` and `conf`
- Files can be included/excluded in the default source sets
- Additional source sets can be configured
- All source sets are listed in the component report for a play application
- All Scala source sets in a Play application are joint-compiled

```gradle
plugins {
    id 'play-application'
}
model {
    components {
        play(PlayApplicationSpec) {
            sources {
                scala {
                    source.include "extraStuff/**"
                }
                resources {
                    source.srcDir "src/assets"
                }
                extraScala(ScalaLanguageSourceSet) {
                    source.srcDir "src/extraScala"
                }
            }
        }
    }
}
```

#### Test cases

- Scala and Jvm source sets (and locations) are visible in the components report
- Can configure additional includes and/or source directories for default `scala` and `resources` source sets
- Can provide an additional Scala source set for a Play application, with dependencies on main sources
- Can provide an additional resources set for a Play application
- Build is incremental:
    - Changed in Scala comment does not result in rebuilding Play app
    - Change in Scala or Java source file triggers recompilation
    - Change in Scala or Java compile properties triggers recompilation
    - Change in Platform triggers recompilation
    - Change in ToolChain triggers recompilation
    - Removal of Scala/Java source files removes classes from the Play app

#### Open issues

- Use `LanguageTransform` from `scala-lang` plugin to configure scala compile task for source sets
- Use source set dependencies to determine of source sets should be joint compiled

### Story: Developer configures template sources for Play application

Add a TwirlSourceSet and permit multiple instances in a Play application

- Twirl sources show up in component report
- Allow Twirl source location to be configured
- Allow additional Twirl source sets to be configured
- Define a generated ScalaSourceSet as the output for each TwirlSourceSet compilation
    - Generated ScalaSourceSet will be one of the Scala compile inputs
    - Generated ScalaSourceSet should not be visible in components report
- Source generation and compilation should be incremental and remove stale outputs.

#### Open issues

- handle non html templates
- Ability for Twirl compiler to prefer Java types in generated sources (i.e. SBT enablePlugins(PlayJava))
- Verify that default imports are configured: https://github.com/playframework/playframework/blob/master/framework/src/build-link/src/main/java/play/TemplateImports.java
- Allow Twirl to be used in a non-play project

### Story: Developer defines routes for Play application

Add a RoutesSourceSet and permit multiple instances in a Play application

- Routes sources show up in component report
- Allow Routes source location to be configured
- Allow additional Routes source sets to be configured
- Define a generated ScalaSourceSet as the output for each RoutesSourceSet compilation
    - Generated ScalaSourceSet will be one of the Scala compile inputs
    - Generated ScalaSourceSet should not be visible in components report
- Source generation and compilation should be incremental and remove stale outputs.

#### Open issues

- handle .routes files
- Ability for Routes compiler to prefer Java types in generated sources (i.e. SBT enablePlugins(PlayJava))

## Feature: Basic support for assets in Play application

### Story: Developer includes static assets in Play application (✔︎)

Any files under `/public` in the application sources are available under `/assets` in the running application.

### Story: Developer includes compiled coffeescript assets in Play application (✔︎)

Add a coffee script plugin as well as JavaScriptSourceSet and CoffeeScriptSourceSets and permit multiple instances.

```gradle
plugins {
    id 'play-application'
    id 'play-coffeescript'
}

model {
    components {
        play(PlayApplicationSpec) {
            sources {
                extraCoffeeScript(CoffeeScriptSourceSet) {
                    sources.srcDir "src/extraCoffeeScript"
                }

                extraJavaScript(JavaScriptSourceSet) {
                    sources.srcDir "src/extraJavaScript"
                }
            }
        }
    }
}
```

- Default coffeescript sourceset should be "app/assets/\*\*/\*.coffee"
- Compiled coffeescript files will be added to the jar under "public"
- Default javascript sourceset should be "app/assets/\*\*/\*.js"
- Processed javascript sourceset should be added to the jar under "public"

#### Test cases
- Coffeescript and javascript sources are visible in the components report
- Coffeescript sources successfully compiled to javascript
- Compiled coffeescript is added to jar under "public"
- Javascript sources are copied directly into jar under "public"
- Can provide additional coffeescript sources
- Can provide additional javascript sources
- Build is incremental:
    - Change in coffeescript source triggers recompile
    - No change in coffeescript source does not trigger a recompile
    - Removal of generated javascript triggers recompile
    - Removal of coffeescript source files removes generated javascript

### Story: Developer uses minified javascript assets in Play application (✔︎)

Extend the basic JavaScript plugin to include minified javascript assets in the Play application.
Use the Google Closure Compiler to produce a minified version of all Javascript assets. The minified files
should be named `<original-name>.min.<original-extension>` (usually the extension will be `.js`).

```gradle
plugins {
    id 'play'
}

model {
    components {
        play(PlayApplicationSpec) {
            sources {
                extraJavaScript(JavaScriptSourceSet) {
                    sources.srcDir "src/extraJavaScript"
                }
            }
        }
    }
}
```

#### Test cases
- Any javascript file in `app/assets` is available in minified form in the app.
- Any javascript file in a configured JavaScriptSourceSet is available in minified form in the app.
- Any compiled coffeeScript source file is available in both non-minified and minified javascript forms.
- Build is incremental:
    - Minifier is not executed when no source inputs have changed
    - Changed javascript source produces changed minified javasript
    - Changed coffeescript source produces changed minified javascript
    - Removal of javascript source removes minified javascript
    - Removal of coffeescript source removes both minified and non-minified javascript

### Story: Documentation and release notes for Play support

Before the support for Play framework is fully usable and can be properly 'released' we need to add documentation and release notes.

## Feature: Developer chooses target Play, Scala and/or Java platform

### Story: Compilation and building is incremental with respect to changes in Play platform

### Story: Build author declares target Play and Scala platform for Play application

```gradle
model {
    components {
        play(PlayApplicationSpec) {
            platform play: "2.3.7", scala: "2.11"
        }
    }
}
```

- If not specified, Play major version number implies Scala version
    - Play 2.2.x -> Scala 2.10
    - Play 2.3.x -> Scala 2.11
- Java version is taken from version executing Gradle

#### Test cases

- For each supported Play version: 2.2.3, 2.3.7
    - Can assemble Play application
    - Can run Play application
    - Can test Play application
- Can build & test multiple Play application variants in single build invocation
    - `gradle assemble` builds all variants
    - `gradle test` tests all variants
- Play version should be visible in components report and dependencies reports.
- Most Play integration tests should be able to run against different supported platforms
    - Developer build should run against single version by default
    - CI should run against all supported versions (using '-PtestAllPlatforms=true')

### Story: Build author declares target Java platform for Play application

```gradle
model {
    components {
        play(PlayApplicationSpec) {
            platform play: "2.3.7", scala: "2.11", java: "1.8"
        }
    }
}
```

### Story: Developer configures target Java and Scala platforms for Jvm library

```gradle
plugins {
    id 'jvm-component'
    id 'scala-lang'
}
model {
    components {
        scalaLib(JvmLibrarySpec) {
            platform java: "1.6", scala: "2.10"
        }
    }
}
```

- Pull `platform(Object)` method up from `PlayApplicationSpec` to `PlatformAwareComponentSpec`
- Allow arbitrary `Map<String, String>` inputs to `platform()`, converting into a `PlatformRequirement`
- Add a `CompositePlatform` type that is returned by `PlatformResolver.resolve()`
    - Allow various typed `PlatformResolver` instances to plugin to the composite `PlatformResolver`
    - When resolving a play platform, return a composite platform with `PlayPlatform`, `ScalaPlatform` and `JavaPlatform` components.
    - When resolving a jvm platform, return a composite platform with `JavaPlatform` and optional `ScalaPlatform` components.
         - Maybe add a `JvmPlatform` that extends `CompositePlatform`, encoding jvm compatibility version.
         - Maybe make `PlayPlatform` extend `CompositePlatform` with a `JvmPlatform` component. Later will have a `BrowserPlatform` component.
- Scala-lang plugin registers a resolver that returns a `ScalaPlatform` for a 'scala' platform requirement

### Story: New target platform DSL for native components

```gradle
model {
    components {
        nativeLib(NativeLibrarySpec) {
            platform os: "windows", arch: "x86"
        }
    }
}
```

## Feature: Basic dependency management for Play

### Story: Developer configures dependencies for Play Application

---

---

# Milestone 2

## Feature: Gradle continuous mode

This story adds a general-purpose mechanism which is able to keep the output of some tasks up-to-date when source files change.
For example, a developer may run `gradle --continuous <tasks>`.

When run in continuous mode, Gradle will execute a build and determine any files that are inputs to that build.
Gradle will then watch for changes to those input files, and re-execute the build when any file changes.

Input files are determined as:
- Files that are inputs to a task but not outputs of some other task
- Files that are inputs to the model

So:

- `gradle --continuous run` would build and run the Play application. When a change to the source files are detected, Gradle would rebuild and
  restart the application.
- `gradle --continuous test run` would build and run the tests and then the Play application. When a change to the source files is detected,
  Gradle would rerun the tests, rebuild and restart the Play application.
- `gradle --continuous test` would build and run the tests. When a source file changes, Gradle would rerun the tests.

Note that for this feature, the implementation will assume that any source file affects the output of every task listed on the command-line.
For example, running `gradle --continuous test run` would restart the application if a test source file changes.

### ~~Story: Add continuous Gradle mode triggered by timer~~

Gradle will be able to start, run a set of tasks and then wait for a retrigger before re-executing the build.

#### Implementation

See spike: https://github.com/lhotari/gradle/commit/969510762afd39c5890398e881a4f386ecc62d75

- Gradle CLI/client process connects to a daemon process as normal.
- Gradle Daemon knows when we're in "continuous mode" and repeats the last build until cancelled.
- Gradle CLI waits for the build to finish (as normal).
- Instead of returning after each build, the daemon goes into a retry loop until cancelled triggered by something.
- Initial implementation will use a periodic timer to trigger the build.
- Add new command-line option (`--continuous`)
    - Add a separate Parameters option
    - Think about how to introduce a new internal replacement for StartParameter
- Decorator for `InProcessBuildExecutor` changes to understand "continuous mode"
    - Similar to what the spike did
    - Build loop delegates to wrapped BuildActionExecuter
    - After build, executor waits for a trigger from somewhere else
- On Ctrl+C, Gradle exits and cancels build.

```
pseudo:

interface TriggerDetails {
    String reason
}
interface TriggerListener {
    void triggered(TriggerDetails)
}
interface Triggerable {
    TriggerDetails waitForTrigger()
}

// In run/execute()
while (not cancelled) {
    delegateExecuter.execute(...)
    triggerable.waitForTrigger()
}

// Triggerable
def waitForTrigger() {
    sync(lock) {
        while(!triggered) {
            lock.wait()
        }
    }
}

def triggered(TriggerDetails) {
    sync(lock) {
        lock.notify()
    }
}
```

#### Test Coverage

- ~~If Gradle build succeeds, we wait for trigger and print some sort of helpful message.~~
- ~~If Gradle build fails, we still wait for trigger.~~
- ~~Configuration errors should be treated in the same way as execution failures.~~
- ~~When "trigger" is tripped, a build runs.~~
- ~~Add coverage for a build that succeeds, then fails, then succeeds (eg a compilation error)~~
- ~~Fail when this is enabled on Java 6 builds, tell the user this is only supported for Java 7+.~~

### ~~Story: Continuous Gradle mode triggered by file change~~

Gradle will be able to start, run a set of tasks and then monitor one file for changes without exiting.  When this file is changed, the same set of tasks will be re-run.

#### Implementation

- Watch project directory (`projectDir`) for changes to trigger re-run
- Add `FileWatchService` that can be given Files to watch
- When files change, mark the file as out of date
- Re-run trigger polls the watch service for changes at some default rate ("quiet period")
- ~~Ignore build/ .gradle/ etc files.~~

#### Test Coverage

- ~~When the project directory files change/are create/are delete, Gradle re-runs using the same set of task selectors.~~

#### Open Issues

N/A

### ~~Story: Continuous Gradle mode triggered by task input changes~~

After performing a build, Gradle will automatically rerun the same logical build if the file system inputs of any task that was executed change.

#### Implementation

1. A logical description of the input files for each task that is executed is captured during the build
2. At the end of the build, Gradle will monitor the file system for changes
3. If any change occurs to a file/directory described by #1, the build will execute again after shutting down all file system listeners

Constraints:

1. There is no way to stop listening for filesystem changes other than use of ctrl-c or similar
2. Changes that after the task executes, but before the build completes and starts watching the file system, are “ignored” (i.e. do not trigger a rebuild)
3. Builds are triggered as soon as a change to a relevant file is noticed (i.e. no quiet period)
4. Continuous mode is not supported by Tooling API (build fails eagerly as unsupported)
5. Symlinks are treated as regular files and changes “behind” symlinks are not respected
6. Only explicit file system inputs are respected (i.e. no consideration given to build logic changes)
7. Changes to `buildSrc` are not respected
8. Only changes to inputs of task that were executed in the immediately preceding build are respected (`A.dependsOn(B.dependsOn(C))` - if C fails, changes to inputs of `A` and `B` are not respected)

#### Test Coverage

##### General

1. ~~Reasonable feedback when using continuous mode (e.g. incubating message, suitable message at end of build)~~
1. ~~If build fails before any task executes, build exits and does not enter continuous mode~~
1. ~~Can trigger rebuild by changing input file to simple task (i.e. basic smoke test)~~
1. ~~Continuous mode utilises class reuse (e.g. same build script class instance is used for subsequent builds)~~
1. ~~Change to non source input file of task with empty source does not trigger build~~

##### Simple Java plugin usage scenarios

For a conventional `apply plugin: 'java'` project:

1. ~~Can successfully build in continuous mode when there is no source (i.e. `src/main/java` does not exist)~~
1. ~~Can successfully build in continuous after source dir `src/main/java` is removed~~
1. ~~Addition of empty directories to `src/main/java` does not trigger rebuild (i.e. source is defined as `include("**/*.java")`)~~
1. ~~After compile failure of Java file, correcting file to be compilable triggers a successful build~~
1. ~~When running `test` in continuous mode:~~
    1. ~~Change to source file causes execution of tests~~
    1. ~~Change to test file causes execution of tests~~
    1. ~~Change to resource file (`src/main/resources`)~~
1. ~~Change to local filesystem compile dependency triggers rebuild (e.g. `lib/some.jar`, not a repo dependency)~~
1. ~~Remove of a local filesystem compile dependency triggers rebuild~~
1. ~~In a multi project, changes to Java source of upstream projects trigger the build of downstream projects~~
1. ~~Project that utilises external repository dependencies can be built in continuous mode (i.e. exercise dependency management layers)~~
1. ~~When main source fails to compile, changes to test source does not trigger build~~
1. ~~Creation of initial source file after initial “empty” build triggers building of jar (i.e. add `/src/main/java/Thing.java`)~~
1. ~~Addition of a local filesystem compile dependency triggers rebuild (e.g. `dependencies { compile fileTree("lib") }`)~~

##### Verifying constraints

1. ~~Continuous mode can be used successfully on a project with a `buildSrc` directory~~
2. ~~Attempting to run a continuous mode build from the Tooling API yields an error immediately~~

##### Edge cases

1. ~~Failure to determine file system inputs for tasks yields reasonable error message (e.g. `javaCompile.src(files( { throw new Exception("!") }))`)~~
1. ~~Task can specify project directory as a task input; changes are respected~~
1. ~~Task can specify root directory of multi project build as a task input; changes are respected~~
1. ~~Continuous mode can be used on reasonable size multi project Java build in conjunction with --parallel~~
1. ~~Can use a symlink as an input file~~
1. ~~Symlinks are not followed for watching purposes (i.e. contents of symlinked directory are not watched)~~

#### Archives

1. ~~With zip task whose contents are directory `src`, adding a new empty directory causes rebuild~~
1. ~~Changes to input zips are respected~~
1. ~~Changes to input tars are respected (compressed and uncompressed)~~

### Story: Continuous build is executed via the Tooling API

This story adds support for executing continuous builds via the Tooling API.
It does not address continually building/providing tooling models.
It also does not improve the general capabilities of continuous mode.

#### Test Coverage

- ~~client executes continuous build that succeeds, then responds to input changes and succeeds~~
- ~~client executes continuous build that succeeds, then responds to input changes and fails, then … and succeeds~~
- ~~client executes continuous build that fails, then responds to input changes and succeeds~~
- ~~client can cancel during execution of a continuous build~~
- ~~client can cancel while a continuous build is waiting for changes~~
- ~~client can receive appropriate logging and progress events for subsequent builds in continuous mode~~
- ~~client receives appropriate error if continuous mode attempted on unsupported platform~~
- ~~logging does not include message to use `ctrl-d` to exit continuous mode~~
- ~~All tooling API clients that support cancellation (>=2.1) can run continuous build~~
- ~~Attempt to run continuous build with tooling api client that does not support cancellation fails eagerly~~
- ~~client can request continuous mode when building a model, but request is effectively ignored~~

### ~~Story: Command line user exits continuous build mode without killing the Gradle process~~

Prior to this story, the only way for a command line user to exit continuous mode is to kill the process.
This story makes the use of continuous mode more effective by allowing better utilisation of warm Gradle daemons.

`ctrl-d` will replace `ctrl-c` as the advertised mechanism for escaping wait state when using continuous build.

#### Test Coverage
- ~~should cancel continuous build by [EOT](http://en.wikipedia.org/wiki/End-of-transmission_character) (ctrl-d)~~
- ~~should cancel build when System.in is closed~~
- ~~should cancel build when System.in contains some other characters, then closes~~
- ~~does not cancel on EOT or System.in closing when not interactive~~
- ~~does not cancel continuous build when other than EOT is entered~~
- ~~can cancel continuous build by EOT after multiple builds~~


### Backlog & Open Issues

- Performance benchmarking
- Responding the changes that occur during the build that affect inputs of already executed tasks
- Responding to changes to build logic implementation:
    - `buildSrc`
    - project build scripts
    - script plugins
    - dynamic plugin dependencies
- Responding to changes to “dynamic” dependencies (i.e. in the `dependencies {}` sense)
- Responding to dynamic inputs to build logic (e.g. properties file read by adhoc user code that externalises build logic)
- Certain inputs might be known to be immutable (e.g. cached repository dependencies have a checksum in their path and will not change, system header files)
- Changes to files behind symlinks are not respected
- Potentially allowing some way to manually trigger a rebuild (e.g. inputs changes before build finished, file system changed was captured - either correctly or incorrectly)

---

## Feature: Keep running Play application up-to-date when source changes

### Story: Play application stays running across continuous builds

This story builds the mechanics necessary to have the play run task launch the Play process without blocking the build.
This allows continuous build to function as normal.
Actually reloading the Play application is out of scope for this story.
The application can still respond to changes to assets that are generated by the build (e.g. processed images).

No public API or functionality is required.
All infrastructure can be internal and be just enough to meet the requirements for running Play.

#### Implementation

    // Gradle service, that outlives a single build (i.e. maybe global)
    // Accessible to tasks via service extraction
    @ThreadSafe
    interface DeploymentRegistry extends Stoppable {
      void register(DeploymentHandle handle);
      <T extends DeploymentHandle> void get(Class<T> handleType, String id);
    }

    interface DeploymentHandle extends Stoppable {
      String getId();
      boolean isRunning();
    }

When continuously building, the play run task, on first invocation, will register a deployment handle for the play process.
Subsequent invocations will do nothing.

#### Test coverage

- ~~Play application stays running across continuous build instances (implicitly: > 1 instances are never launched)~~
- ~~Build failure prior to launching play app (e.g. compile failure) does not prevent app from being launched on subsequent build~~
- ~~Play application is shutdown when build is cancelled~~
- ~~PlayRun task blocks when not continuously building~~
- ~~Can be used from Tooling API, deployment is stopped when build is cancelled~~
- ~~Two projects in multiproject build can start deployments~~

#### Open Issues

- Need to figure out what to do with deployment handles when various project/model changes take place in between builds:
  - Binary name changes (currently not possible)
  - Platform changes (i.e. need to stop any deployment handles from other platforms before starting the new one)
  - Project name changes
  - PlayRun task config changes (e.g. fork options)

### Story: Play application is reloaded when changes are made to local source

This story adds integration with Play's mechanics to reloading the application implementation.
No changes to continuous build are required.
Builds are triggered as soon as changes are detected.

#### Implementation

On reload, the play run task retrieves the deployment handle (i.e. by Play specific subtype) and “triggers a reload”.
The worker protocol needs to be slightly expanded to allow communication of the new classpath/reload request.
Existing BuildLink adapter can be used with minor modifications.



#### Test coverage

- ~~Changes to source are reflected in running play app~~
   - ~~test changes to different types of source:~~
     - ~~controller (java/scala)~~
     - ~~routes~~
     - ~~model (java/scala)~~
     - ~~views (twirl)~~
     - ~~javascript~~
     - ~~coffeescript~~
     - ~~css~~
- Reload is not triggered if dependency of play run task fails

### Story: Reloading changes in multi-project Play application

#### Implementation

The Play BuildLink solution returns a new classloader that contains the "changing" application resources. The non-changing application resources are part of the parent classloader for the classloader returned by the BuildLink implementation.
In a multi-project Play application, the "changing" application resources are considered to be the application jar of the current project and all dependencies that are of a Project component type. All other resources in the runtime classpath are part of the "non-changing" resources and handled by the parent classloader.
 - dependencies of project component type will be filtered by checking `ResolvedArtifact.getId().getComponentIdentifier() instanceof ProjectComponentIdentifier`

#### Test coverage
- Changes to source are reflected in running play app
  - test changes to primary app and submodules

### Story: Scala compile process is reused across continuous build instances

This is a performance optimization to the implementation of Play reload, that decreases compile times by reusing compiler daemons.
This story also supersedes the `BuildSession` concept added in earlier stories with a service registry scope serving the same purpose.

#### Implementation

- Introduce `BuildSessionScopeServices`
- Move `CompilerDaemonManager` to session scope

#### Test coverage

TBD.

### Backlog & Open Issues

- Debugging Play application that is run with playRun
  - currently there isn't a way to add debugging JVM arguments to the worker process
- PlayRun uses org.gradle.api.tasks.compile.BaseForkOptions for forkOptions. That class is documented to hold forked compiler JVM arguments.

## Play 2.4.x support

### Story: Add basic support for Play 2.4.x

The Gradle Play support has a structure for supporting multiple Play major versions. Currently 2.2.x and 2.3.x are supported.
Support for 2.4.x can be done with the same structure.
- Add PlayMajorVersion enum for 2.4.x
- Add 2.4.x specific adaptors for
 - routes compiler
 - twirl compiler
 - play run (starting development server)

Test coverage:
- Add Play 2.4.0 to the target coverage of PlayMultiVersionIntegrationTest so that the same level of test coverage is applied to test Play 2.4.0

### Story: Add support for configuring Play 2.4.x routes compiler type (injected/static)

Play 2.4 introduces a new configuration option `routesGenerator` for the route compiler.
```
// Play provides two styles of routers, one expects its actions to be injected, the
// other, legacy style, accesses its actions statically.
routesGenerator := InjectedRoutesGenerator
```
There are currently two different types of routesGenerators: `play.routes.compiler.InjectedRoutesGenerator` and `play.routes.compiler.StaticRoutesGenerator` .
StaticRoutesGenerator creates the legacy style access to "object" type Scala controllers. InjectedRoutesGenerator expects controllers to be "class" type in Scala.
In Play Java, the legacy style uses static methods and the newer injected style uses normal non-static methods.
Play uses Guice to do autowiring in the new style.

Test coverage:
- add Play 2.4.x integration test application that uses injected routes compiler and non-static style of controllers in Scala and Java.

### Open Issues

- The routes compiler contains a setting `playAggregateReverseRoutes` in Play 2.4.x. The definition is "A list of projects that reverse routes should be aggregated from.".
 - Play manual: [Aggregating Reverse routers](https://github.com/playframework/playframework/blob/2.4.x/documentation/manual/detailedTopics/build/AggregatingReverseRouters.md)
 - [Routes compilation sbt task in 2.3.x](https://github.com/playframework/playframework/blob/2.3.x/framework/src/sbt-plugin/src/main/scala/PlaySourceGenerators.scala#L20)
 - [Routes compilation sbt task in 2.4.x](https://github.com/playframework/playframework/blob/2.4.x/framework/src/sbt-plugin/src/main/scala/play/sbt/routes/RoutesCompiler.scala#L48)


### Story: Build of pending changes is deferred until reload is requested by Play application

This story aligns the behavior of the Play reload support closer to SBT's implementation.
Instead of building as soon as local changes are noticed, building will be deferred until the next request to the Play application.
This improves usability by not doing the work of _applying_ changes until the developer is ready to test them.

*TBD - value/priority of feature is not yet understood*.

## Feature: Developer views compile and other build failures in Play application

### Story: Developer views build failure message in Play application

Adapt a generic build failure exception to a `PlayException` that renders the exception message.

### Story: Developer views Java and Scala compilation failure in Play application

Adapt compilation failures so that the failure and content of the failing file is displayed in the Play application.

### Story: Developer views Asset compilation failures in Play application

Failures in CoffeeScript compilation are rendered with content of the failing file.
This mechanism will be generally applicable to custom asset compilation tasks.

### Story: Developer views build failure stack trace in Play application

## Feature: Resources are built on demand when running Play application

When running a Play application, start the application without building any resources. Build these resources only when requested
by the client.

- On each request, check whether the task which produces the requested resource has been executed or not. If not, run the task synchronously
  and block until completed.
- Include the transitive input of these tasks as inputs to the watch mechanism, so that further changes in these source files will
  trigger a restart of the application at the appropriate time.
- Failures need to be forwarded to the application for display.

## Feature: Long running compiler daemon

Reuse the compiler daemon across builds to keep the Scala compiler warmed up. This is also useful for the other compilers.

### Implementation

- Maintain a registry of compiler daemons in ~/.gradle
- Daemons expire some time after build, with much shorter expiry than the build daemon.
- Reuse infrastructure from build daemon.


# Later milestones

## Feature: Developer adds default Play repositories to build script

This allows the developer to easily add repositories to the build script using a convenience extension on the RepositoryContainer object.

```gradle
buildscript {
    repositories {
        gradlePlay()
    }
}
```

This should point to a virtual repository (play-public) at gradle.repo.org that's backed by the default repositories required for play functionality.
Currently the following repositories would be required:
- https://repo.typesafe.com/typesafe/maven-releases (play support)
- https://repo.gradle.org/gradle/javascript-public (coffeescript and other javascript artifacts)

#### Test Cases
- Can build a basic play application using the convenience extension to specify repositories
- Can build a coffeescript-enabled play application using the convenience extension to specify repositories

## Feature: Build author provides plugin for asset processing in Play application (Javascript, LESS, CoffeeScript)

Extend the standard build lifecycle to compile the front end assets to CSS and Javascript.

- Provide some built-in implementations of asset plugins
    - Coffeescript -> Javascript
    - LESSCSS -> CSS
    - Javascript > Javascript via Google Closure
    - Javascript minification, requirejs optimization
- For built-in asset plugins
    - Build on public API
    - Include the compiled assets in the Jar
    - Define source sets for each type of source file
    - Compilation should be incremental and remove stale outputs
    - Expose some compiler options

### Implementation

JavaScript language plugin:

- Defines JavaScript library component and associated JavaScript bundle binary.
- Defines JavaScript source set type (a JavaScript bundle and JavaScript source set should be usable in either role).
- Defines transformation from JavaScript source set to JavaScript bundle.

CSS language plugin:

- Defines CSS library component and associated CSS bundle binary.
- Defines CSS source set type (a CSS bundle and CSS source set should be usable in either role).
- Defines transformation from CSS source set to CSS bundle.

CoffeeScript plugin:

- Defines CoffeeScript source set type and transformation to JavaScript bundle.

LESSCSS plugin:

- Defines LESSCSS source set type and transformation to CSS bundle.

Google Closure plugin:

- Defines transformation from JavaScript source set to JavaScript bundle.

Play plugin:

- Defines JavaScript and CSS components for the Play application.
- Wires in the appropriate outputs to assemble the Jar.

### Open issues

- Integration with existing Gradle javascript plugins.

## Documentation

- Migrating an SBT based Play project to Gradle
- Writing Gradle plugins that extend the base Play plugin

## Native integration with Specs 2

Introduce a test integration which allows Specs 2 specifications to be executed directly by Gradle, without requiring the use of the Specs 2
JUnit integration.

- Add a Specs 2 plugin
- Add some Specs 2 options to test tasks
- Detect specs 2 specifications and schedule for execution
- Execute specs 2 specifications using its API and adapt execution events

Note: no changes to the HTML or XML test reports will be made.

## Developer runs Scala interactive console

Allow the Scala interactive console to be launched from the command-line.

- Build the project's main classes and make them visible via the console
- Add support for client-side execution of actions
- Model the Scala console as a client-side action
- Remove console decoration prior to starting the Scala console

## Scala code quality plugins

- Scalastyle
- SCCT

## Javascript plugins

- Compile Dust templates to javascript and include in the web application image

## Bootstrap a new Play project

Extend the build init plugin so that it can bootstrap a new Play project, producing the same output as `activator new` except with a Gradle build instead of
an SBT build.

## Publish Play application to a binary repository

Allow a Play application distribution to be published to a binary repository.

Some candidates for later work:

- Improve the HTML test report to render a tree of test executions, for better reporting of Specs 2 execution (and other test frameworks)
- Support the new Java and Scala language plugins
- Improve watch mode so that only those tasks affected be a given change are executed

## Further features

- Internal mechanism for plugin to inject renderer(s) into components report.
- Model language transformations, and change Play support to allow a Play application to take any JVM language as input.
- Declare dependencies on other Java/Scala libraries
- Control joint compilation of sources based on source set dependencies.
- Build multiple variants of a Play application.
- Generate an application install, eg with launcher scripts and so on.
